/*
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1999,2005  All rights reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.almaden;

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

/**
 * 
 * Send email.
 * 
 * @author <A HREF="mailto:jussi@almaden.ibm.com">Jussi Myllymaki</a>
 * @version $Revision: 1 $ $Date: 5/09/05 5:41p $
 */

public class SendMail {

	/**
	 * Send email to recipient. Attach files to email message if requested.
	 * @param	gateway			Address of email gateway.
	 * @param	sender			Sender's email address.
	 * @param	senderPersonal	Sender's personal name.
	 * @param	recipient		Recipient of the email.
	 * @param	subject			Subject of the message.
	 * @param	body			Body of the message.
	 * @param	attachments		Vector of attachments (File or String objects).
	 * @param	attachmentNames	File names (Content-Disposition) of attachments.
	 */
	public static synchronized void send(String gateway, String sender, String senderPersonal,
					     String recipient, String subject, String body, Vector attachments,
					     Vector attachmentNames)
		throws Exception {

//		log.print(log.IMPORTANT, getClass().getName(), "Sending email to " + recipient);

		Properties props = System.getProperties();
		props.put("mail.smtp.host", gateway);
		Session session = Session.getDefaultInstance(props, null);
//		session.setDebug(log.getLevel() >= log.DETAIL);

		MimeMessage msg = new MimeMessage(session);
		msg.setFrom(new InternetAddress(sender, senderPersonal));
		InternetAddress[] address = { new InternetAddress(recipient) };
		msg.setRecipients(Message.RecipientType.TO, address);
		msg.setSubject(subject);

		// create and fill the message body
		MimeBodyPart mbp = new MimeBodyPart();
		mbp.setText(body);

		// create the Multipart and its parts to it
		Multipart mp = new MimeMultipart();
		mp.addBodyPart(mbp);

		for (int i = 0; i < attachments.size(); i++) {

			// append attachment to the message
			Object att = attachments.elementAt(i);
//			log.print(log.DETAIL, getClass().getName(), "Attaching content " + (String)attachmentNames.elementAt(i));
			MimeBodyPart attachment = new MimeBodyPart();

			if (att instanceof File) {
				File fatt = (File)att;
				FileDataSource fds = new FileDataSource(fatt);
				attachment.setDataHandler(new DataHandler(fds));
				attachment.setFileName((String)attachmentNames.elementAt(i));
			} else if (att instanceof String) {
				String satt = (String)att;
				attachment.setText(satt);
				attachment.setFileName((String)attachmentNames.elementAt(i));
			} else if (att instanceof Reader) {
				Reader ratt = (Reader)att;
				StringBuffer sbuf = new StringBuffer();
				char[] buf = new char [16384];
				int charsread = 0;
				while ((charsread = ratt.read(buf)) > 0) {
					sbuf.append(buf, 0, charsread);
				}
				attachment.setText(sbuf.toString());
				attachment.setFileName((String)attachmentNames.elementAt(i));
			} else
				throw new IllegalArgumentException("Invalid attachment type");

			mp.addBodyPart(attachment);

		}

		// add the Multipart to the message
		msg.setContent(mp);

		// set the Date: header
		msg.setSentDate(new java.util.Date());
	    
		// send the message
		Transport.send(msg);

//		log.print(log.IMPORTANT, getClass().getName(), "Done sending email to " + recipient);

	}

	/**
	 * Main program.
	 */
	public static void main(String argv[]) throws Exception {

		if (argv.length < 6) {
			printUsage();
			System.exit(1);
		}

		String gateway = argv[0];
		String sender = argv[1];
		String senderPersonal = argv[2];
		String recipient = argv[3];
		String subject = argv[4];
		String body = argv[5];
		
		Vector attachments = new Vector();
		Vector attachmentNames = new Vector();
		for (int i = 6; i < argv.length; i++) {
			System.out.println("Attaching file " + argv[i]);
			attachments.add(new File(argv[i]));
			attachmentNames.add(argv[i]);
		}
		
		send(gateway, sender, senderPersonal, recipient, subject, body, attachments, attachmentNames);

		System.exit(0);

	}

	/**
	 * Show usage.
	 */	
	protected static void printUsage() {

		System.out.println("Usage: java SendMail gateway sender senderPersonal recipient subject body [attachment ...]");

	}

}

/**
 * $History: SendMail.java $
 * 
 * *****************  Version 1  *****************
 * User: Jussi        Date: 5/09/05    Time: 5:41p
 * Created in $/DB2SOAP/Development/SQL2Excel/com/ibm/almaden
 * Initial revision. Copied some content from Andes project.
 * 
 * *****************  Version 1  *****************
 * User: Jussi        Date: 8/07/02    Time: 9:53a
 * Created in $/ANDES/Server
 * Initial revision.
 */
